[llvm-commits] [llvm] r131940 - /llvm/trunk/lib/CodeGen/AsmPrinter/DwarfDebug.cpp

Devang Patel dpatel at apple.com
Mon May 23 17:22:25 PDT 2011


Author: dpatel
Date: Mon May 23 19:22:25 2011
New Revision: 131940

URL: http://llvm.org/viewvc/llvm-project?rev=131940&view=rev
Log:
Fix debug info for blocks' variable.

Modified:
    llvm/trunk/lib/CodeGen/AsmPrinter/DwarfDebug.cpp

Modified: llvm/trunk/lib/CodeGen/AsmPrinter/DwarfDebug.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/CodeGen/AsmPrinter/DwarfDebug.cpp?rev=131940&r1=131939&r2=131940&view=diff
==============================================================================
--- llvm/trunk/lib/CodeGen/AsmPrinter/DwarfDebug.cpp (original)
+++ llvm/trunk/lib/CodeGen/AsmPrinter/DwarfDebug.cpp Mon May 23 19:22:25 2011
@@ -2590,12 +2590,27 @@
         unsigned i = 0;
         Asm->OutStreamer.AddComment("Loc expr size");
         if (N >= 2 && DV.getAddrElement(0) == DIBuilder::OpPlus) {
-          // If first address element is OpPlus then emit
-          // DW_OP_breg + Offset instead of DW_OP_reg + Offset.
-          MachineLocation Loc(Entry.Loc.getReg(), DV.getAddrElement(1));
-          Asm->EmitInt16(Asm->getDwarfRegOpSize(Loc) + N - 2);
-          Asm->EmitDwarfRegOp(Loc);
-          i = 2;
+          if (Entry.Loc.getOffset()) {
+            unsigned Size = Asm->getDwarfRegOpSize(Entry.Loc);
+            unsigned OffsetSize = 
+              MCAsmInfo::getSLEB128Size(DV.getAddrElement(1));
+            // breg + deref + plus + offset
+            Asm->EmitInt16(Size + 1 + 1 + OffsetSize + N - 2);
+            i = 2;
+            Asm->EmitDwarfRegOp(Entry.Loc);
+            Asm->OutStreamer.AddComment("DW_OP_deref");
+            Asm->EmitInt8(dwarf::DW_OP_deref);
+            Asm->OutStreamer.AddComment("DW_OP_plus_uconst");
+            Asm->EmitInt8(dwarf::DW_OP_plus_uconst);
+            Asm->EmitSLEB128(DV.getAddrElement(1));
+          } else {
+            // If first address element is OpPlus then emit
+            // DW_OP_breg + Offset instead of DW_OP_reg + Offset.
+            MachineLocation Loc(Entry.Loc.getReg(), DV.getAddrElement(1));
+            Asm->EmitInt16(Asm->getDwarfRegOpSize(Loc) + N - 2);
+            Asm->EmitDwarfRegOp(Loc);
+            i = 2;
+          }
         } else {
           Asm->EmitInt16(Asm->getDwarfRegOpSize(Entry.Loc) + N);
           Asm->EmitDwarfRegOp(Entry.Loc);





More information about the llvm-commits mailing list